/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2010, Digium, Inc.
 *
 * Russell Bryant <russell@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! 
 * \file
 * \brief Translation Test
 *
 * \author Russell Bryant <russell@digium.com>
 * 
 * \ingroup tests
 */

/*** MODULEINFO
	<depend>TEST_FRAMEWORK</depend>
 ***/

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 272531 $")

#include "asterisk/utils.h"
#include "asterisk/module.h"
#include "asterisk/test.h"
#include "asterisk/translate.h"

AST_TEST_DEFINE(translate_test)
{
	static unsigned char buf[AST_FRIENDLY_OFFSET + 640] = { 0, };
	struct ast_frame f = {
		.frametype = AST_FRAME_VOICE,
		.subclass = AST_FORMAT_SLINEAR16,
		.samples = 320,
		.offset = AST_FRIENDLY_OFFSET,
		.datalen = 640,
		.data.ptr = buf + AST_FRIENDLY_OFFSET,
	};
	struct ast_trans_pvt *trans = NULL;
	struct ast_frame *f_new = NULL;
	enum ast_test_result_state res = AST_TEST_FAIL;

	switch (cmd) {
	case TEST_INIT:
		info->name = "translate";
		info->category = "main/translate/";
		info->summary = "simple ast_translate() unit test";
		info->description = "Perform a basic test of ast_translate().";
		return AST_TEST_NOT_RUN;
	case TEST_EXECUTE:
		break;
	}

	trans = ast_translator_build_path(AST_FORMAT_SLINEAR, AST_FORMAT_SLINEAR16);
	if (!trans) {
		ast_test_status_update(test, "Failed to created translation path from slin16 to slin\n");
		goto cleanup;
	}

	f_new = ast_translate(trans, &f, 0);
	if (!f_new) {
		ast_test_status_update(test, "ast_translate() failed.\n");
		goto cleanup;
	}

	if (f_new->frametype != AST_FRAME_VOICE) {
		ast_test_status_update(test, "Unexpected frametype '%d' \n", f_new->frametype);
		goto cleanup;
	}

	if (f_new->subclass != AST_FORMAT_SLINEAR) {
		ast_test_status_update(test, "Unexpected output format '%d' \n", f_new->subclass);
		goto cleanup;
	}

	if (f_new->samples != 160) {
		ast_test_status_update(test, "Unexpected samples '%d'\n", f_new->samples);
		goto cleanup;
	}

	if (f_new->datalen != 320) {
		ast_test_status_update(test, "Unexpected datalen '%d'\n", f_new->datalen);
		goto cleanup;
	}

	res = AST_TEST_PASS;

cleanup:
	if (trans) {
		ast_translator_free_path(trans);
	}
	if (f_new) {
		ast_frfree(f_new);
	}

	return res;
}

static int unload_module(void)
{
	AST_TEST_UNREGISTER(translate_test);
	return 0;
}

static int load_module(void)
{
	AST_TEST_REGISTER(translate_test);
	return AST_MODULE_LOAD_SUCCESS;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Skeleton (sample) Test");
